unit UfrmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry,
  StdCtrls, Menus;

type
  TfrmMain = class(TForm)
    PopupMenu1: TPopupMenu;
    cmNormal: TMenuItem;
    cmEmboss: TMenuItem;
    cmBlur: TMenuItem;
    Transparentv112000CyLogSoftware1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    cmAbout: TMenuItem;
    cmExit: TMenuItem;
    dlgColor: TColorDialog;
    N3: TMenuItem;
    cmSelectColor: TMenuItem;
    cmMonochrome: TMenuItem;
    cmDisplace: TMenuItem;
    cmTint: TMenuItem;
    cm3DBorder: TMenuItem;
    cmIncreaseWidth: TMenuItem;
    cmDecreaseWidth: TMenuItem;
    N4: TMenuItem;
    cmIncHeight: TMenuItem;
    cmDecHeight: TMenuItem;
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure cmNormalClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure cmExitClick(Sender: TObject);
    procedure cmAboutClick(Sender: TObject);
    procedure cmSelectColorClick(Sender: TObject);
    procedure cmTintClick(Sender: TObject);
    procedure cm3DBorderClick(Sender: TObject);
    procedure cmIncreaseWidthClick(Sender: TObject);
    procedure cmDecreaseWidthClick(Sender: TObject);
    procedure cmDecHeightClick(Sender: TObject);
    procedure cmIncHeightClick(Sender: TObject);
  private
    { Private declarations }
  public
      MBDown  : boolean;
      MBX,MBY : integer;
      Filter  : integer;

      b               : TBitmap;
      BackBMPCreated  : boolean;
      BackBMPFilename : string;
      BackBMPStyle    : integer;
      BackBMPTile     : integer;

      memDC       : HDC;
      bi          : TBitmapInfo;
      
      RowOffset   : integer;  // every row starts at exactly one full DWORD
      FirstTime   : boolean;

      procedure MakeFilter ( n : byte; b : TBitmap );
      procedure GetBitmap;
      procedure GetBitmapFromBackground;    // this is the old one which took the background.
  end;
  TRGBArray = array[0..10000000] of byte;

const
    B_ = 0;
    G_ = 1;
    R_ = 2;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure DrawPartBMP ( canv : TCanvas; X,Y,W,H,XRel,YRel : integer; bmp : Graphics.TBitmap );
var
    MemDC : HDC;

begin
    MemDC := CreateCompatibleDC(canv.Handle);
    SelectObject (MemDC,bmp.Handle);
    BitBlt (canv.Handle,X,Y,W,H,MemDC,XRel,YRel,SRCCopy);
    DeleteDC (MemDC);
end;    

procedure TfrmMain.MakeFilter ( n : byte; b : TBitmap );
const
    weight : array[-1..1,-1..1] of double = ((0.7,1.0,0.7),
                                             (1.0,1.0,1.0),
                                             (0.7,1.0,0.7));

var
    res             : integer;
    Buf1,Buf2,helpP : pointer;
    x,q1,q2         : ^TRGBArray;
    sumR,sumG,sumB,
    divider         : double;
    off             : array[-1..1,-1..1] of integer;

    i,j,k,l         : integer;
    off1,off2       : integer;
    r1,g1,b1,
    new,new1,new2   : integer;
    f               : real;

    ni,nj,cx,cy     : integer;
    ang,radius      : double;

    _R,_G,_B        : integer;

    offRow,
    offR,offG,offB  : integer;

begin
    FillChar (bi,SizeOf(bi),0);
    bi.bmiHeader.biSize := $28;
    res := GetDIBits (memDC,b.Handle,0,0,nil,bi,DIB_RGB_COLORS);
    if res = 0 then
        Exit;                               // Cannot get Bitmap Info with GetDIBits
    if bi.bmiHeader.biBitCount <> 24 then
        Exit;                               // This is not a 24-bit color Bitmap. Conversion not done properly!

    GetMem (Buf1,bi.bmiHeader.biSizeImage);
    GetMem (Buf2,bi.bmiHeader.biSizeImage);

    res := GetDIBits (memDC,b.Handle,0,bi.bmiHeader.biHeight,Buf1,bi,DIB_RGB_COLORS);
    if res = 0 then
        Exit;
    RowOffset := bi.bmiHeader.biSizeImage div bi.bmiHeader.biHeight;

    _R := (dlgColor.Color and $0000FF);
    _G := (dlgColor.Color and $00FF00) shr 8;
    _B := (dlgColor.Color and $FF0000) shr 16;
    // filtering
    case n of
    0 : if cmTint.Checked then
        begin
            x := Buf1;
            for j := 0 to bi.bmiHeader.biHeight-1 do
            begin
                offRow := j*RowOffset;
                for i := 0 to bi.bmiHeader.biWidth-1 do
                begin
                    offR := i*3+R_+offRow;
                    offG := i*3+G_+offRow;
                    offB := i*3+B_+offRow;
                    x[offR] := (x[offR] + _R) div 2;
                    x[offG] := (x[offG] + _G) div 2;
                    x[offB] := (x[offB] + _B) div 2;
                end;
            end;
        end;
    1 : begin
            x := Buf1;
            for j := 0 to bi.bmiHeader.biHeight-1 do
            begin
                offRow := j*RowOffset;
                for i := 0 to bi.bmiHeader.biWidth-1 do
                begin
                    offR := i*3+R_+offRow;
                    offG := i*3+G_+offRow;
                    offB := i*3+B_+offRow;
                    r1 := x[offR];
                    g1 := x[offG];
                    b1 := x[offB];
                    new := Round(r1*0.3+g1*0.3+b1*0.4) mod 256;
                    if cmTint.Checked then
                    begin
                        x[offR] := (new + _R) div 2;
                        x[offG] := (new + _G) div 2;
                        x[offB] := (new + _B) div 2;
                    end
                    else
                    begin
                        x[offR] := new;
                        x[offG] := new;
                        x[offB] := new;
                    end;
                end;
            end;
        end;
    2 : begin
            x := Buf1;
            for j := 0 to bi.bmiHeader.biHeight-2 do
            begin
                offRow := j*RowOffset;
                for i := bi.bmiHeader.biWidth-1 downto 1 do
                begin
                    offR := i*3+R_+offRow;
                    offG := i*3+G_+offRow;
                    offB := i*3+B_+offRow;
                    r1 := x[offR];
                    g1 := x[offG];
                    b1 := x[offB];
                    new1 := Round(r1*0.3+g1*0.3+b1*0.4) mod 256;
                    r1 := 255-x[((i-1)*3)+R_+(j+1)*RowOffset];
                    g1 := 255-x[((i-1)*3)+G_+(j+1)*RowOffset];
                    b1 := 255-x[((i-1)*3)+B_+(j+1)*RowOffset];
                    new2 := Round(r1*0.3+g1*0.3+b1*0.4) mod 256;

                    if cmTint.Checked then
                    begin
                        x[offR] := (new1+new2+_R) div 3;
                        x[offG] := (new1+new2+_G) div 3;
                        x[offB] := (new1+new2+_B) div 3;
                    end
                    else
                    begin
                        x[offR] := (new1+new2) div 2;
                        x[offG] := (new1+new2) div 2;
                        x[offB] := (new1+new2) div 2;
                    end;
                end;
           end;
           for j := 0 to bi.bmiHeader.biHeight-1 do
           begin
               i := bi.bmiHeader.biWidth-1;
               x[R_+j*RowOffset] := x[(i*3)+R_+j*RowOffset];
               x[G_+j*RowOffset] := x[(i*3)+G_+j*RowOffset];
               x[B_+j*RowOffset] := x[(i*3)+B_+j*RowOffset];
           end;
           for i := 0 to bi.bmiHeader.biWidth-2 do
           begin
               j := bi.bmiHeader.biHeight-1;
               x[(i*3)+R_+j*RowOffset] := x[(i*3)+R_];
               x[(i*3)+G_+j*RowOffset] := x[(i*3)+G_];
               x[(i*3)+B_+j*RowOffset] := x[(i*3)+B_];
           end;
        end;
    3 : begin
            q1 := Buf1;
            q2 := Buf2;
            for j := 0 to bi.bmiHeader.biHeight-1 do
                for i := 0 to bi.bmiHeader.biWidth-1 do
                begin
                    sumR := 0;
                    sumG := 0;
                    sumB := 0;
                    divider := 0;
                    for k := -1 to 1 do
                        for l := -1 to 1 do
                            if (i+k >= 0) and (i+k < bi.bmiHeader.biWidth) and
                               (j+l >= 0) and (j+l < bi.bmiHeader.biHeight) then
                            begin
                                off[k,l] := ((i+k)*3)+(j+l)*RowOffset;
                                sumR := sumR+weight[k,l]*q1[off[k,l]+R_];
                                sumG := sumG+weight[k,l]*q1[off[k,l]+G_];
                                sumB := sumB+weight[k,l]*q1[off[k,l]+B_];
                                divider := divider+weight[k,l];
                            end;
                    if cmTint.Checked then
                    begin
                        q2[off[0,0]+R_] := (Round(sumR/divider)+_R) div 2;
                        q2[off[0,0]+G_] := (Round(sumG/divider)+_G) div 2;
                        q2[off[0,0]+B_] := (Round(sumB/divider)+_B) div 2;
                    end
                    else
                    begin
                        q2[off[0,0]+R_] := Round(sumR/divider);
                        q2[off[0,0]+G_] := Round(sumG/divider);
                        q2[off[0,0]+B_] := Round(sumB/divider);
                    end;
                end;
            helpP := Buf2;
            Buf2 := Buf1;
            Buf1 := helpP;
        end;
    4 : begin
            q1 := Buf1;
            q2 := Buf2;
            for j := 0 to bi.bmiHeader.biHeight-1 do
                for i := 0 to bi.bmiHeader.biWidth-1 do
                begin
                    repeat
                        k := i+Random(5)-2;
                    until (k >= 0) and (k <= bi.bmiHeader.biWidth-1);
                    repeat
                        l := j+Random(5)-2;
                    until (l >= 0) and (l <= bi.bmiHeader.biHeight-1);
                    off1 := (i*3)+j*RowOffset;
                    off2 := (k*3)+l*RowOffset;
                    if cmTint.Checked then
                    begin
                        q2[off1+R_] := (q1[off2+R_]+_R) div 2;
                        q2[off1+G_] := (q1[off2+G_]+_G) div 2;
                        q2[off1+B_] := (q1[off2+B_]+_B) div 2;
                    end
                    else
                    begin
                        q2[off1+R_] := q1[off2+R_];
                        q2[off1+G_] := q1[off2+G_];
                        q2[off1+B_] := q1[off2+B_];
                    end;
                end;
           helpP := Buf2;
           Buf2 := Buf1;
           Buf1 := helpP;
        end;
    end;
    if cm3DBorder.Checked then
    begin
        x := Buf1;
        for j := 0 to bi.bmiHeader.biHeight-1 do
            for i := 0 to bi.bmiHeader.biWidth-1 do
            begin
                f := 1.0;
                if (i = 1) or (j = bi.bmiHeader.biHeight-2) then
                    f := 1.5
                else
                    if (i = bi.bmiHeader.biWidth-2) or (j = 1) then
                        f := 0.75;
                if f <> 1.0 then
                begin
                    r1 := Round(f*x[(i*3)+R_+j*RowOffset]);
                    g1 := Round(f*x[(i*3)+G_+j*RowOffset]);
                    b1 := Round(f*x[(i*3)+B_+j*RowOffset]);
                    if f > 1 then
                    begin
                        if r1 > 255 then
                            r1 := 255;
                        if g1 > 255 then
                            g1 := 255;
                        if b1 > 255 then
                            b1 := 255;
                    end;
                    x[(i*3)+R_+j*RowOffset] := r1;
                    x[(i*3)+G_+j*RowOffset] := g1;
                    x[(i*3)+B_+j*RowOffset] := b1;
                end;
            end;
    end;
    // Now we should paste to Bitmap

    res := SetDIBitsToDevice (b.Canvas.Handle,0,0,bi.bmiHeader.biwidth,bi.bmiHeader.biHeight,
                              0,0,0,bi.bmiHeader.biHeight,Buf1,bi,DIB_RGB_COLORS);
    FreeMem (Buf1,bi.bmiHeader.biSizeImage);
    FreeMem (Buf2,bi.bmiHeader.biSizeImage);
end;

procedure TfrmMain.GetBitmap;
var
    hmyrgn,
    hrgn,
    htmprgn,htmprgn2,
    hsumrgn           : THandle;
    rc, wrc           : TRect;
    hdc,hwnd          : THandle;

begin
    if BackBMPCreated then
        b.Free;
    b := TBitmap.Create;
    b.Width := ClientWidth;
    b.Height := ClientHeight;
    b.PixelFormat := pf24bit;

    hmyrgn   := CreateRectRgn(0, 0, 0, 0);
    hsumrgn  := CreateRectRgn(0, 0, 0, 0);
    htmprgn  := CreateRectRgn(0, 0, 0, 0);
    htmprgn2 := CreateRectRgn(0, 0, 0, 0);
    hrgn     := CreateRectRgn(0, 0, 0, 0);

    GetWindowRect(Handle, wrc);
    if (GetWindowRgn(Handle, hmyrgn) = ERROR) then
        hmyrgn := CreateRectRgn(wrc.left, wrc.top, wrc.right, wrc.bottom)
    else
        OffsetRgn(hmyrgn, wrc.left, wrc.top);

    rc.Left   := wrc.Left + 10000;
    rc.Right  := wrc.Right + 10000;
    rc.Top    := wrc.Top + 10000;
    rc.Bottom := wrc.Bottom + 10000;
//    ShowWindow(Handle, SW_HIDE);
    MoveWindow(Handle, rc.Left, rc.Top, rc.Right-rc.Left, rc.Bottom-rc.Top, TRUE);

    hwnd := Handle;
    while true do
    begin
        hwnd := GetNextWindow(hwnd, GW_HWNDNEXT);
        if (hwnd = 0) then
            break;
        if (IsWindowVisible(hwnd)) then
        begin
            GetWindowRect(hwnd, rc);
            if (GetWindowRgn(hwnd, hrgn) = ERROR) then
                hrgn := CreateRectRgn(rc.Left, rc.Top, rc.Right, rc.Bottom)
            else
                OffsetRgn(hrgn, rc.Left, rc.top);

            if (CombineRgn(htmprgn, hmyrgn, hrgn, RGN_AND) <> NULLREGION) then
            begin
                if (CombineRgn(htmprgn2, htmprgn, hsumrgn, RGN_DIFF) <> NULLREGION) then
                begin
                    OffsetRgn(htmprgn2, -rc.Left, -rc.Top);

                    RedrawWindow(hwnd, nil, htmprgn2, RDW_FRAME or RDW_INVALIDATE or RDW_UPDATENOW or
                                                       RDW_ALLCHILDREN or RDW_ERASENOW or RDW_INTERNALPAINT);
                    CombineRgn(hsumrgn, hsumrgn, htmprgn, RGN_OR);
                end;
            end;
        end;
    end;

    hdc := GetDC(0);
    BitBlt (b.Canvas.Handle, 0, 0, ClientWidth, ClientHeight, hdc, wrc.Left, wrc.Top, SRCCOPY);
    ReleaseDC(0, hdc);

    MakeFilter (Filter,b);

    b.Canvas.Pen.Color := clBlack;
    b.Canvas.MoveTo (0,0);
    b.Canvas.LineTo (b.Width-1,0);
    b.Canvas.LineTo (b.Width-1,b.Height-1);
    b.Canvas.LineTo (0,b.Height-1);
    b.Canvas.LineTo (0,0);

    MoveWindow(Handle, wrc.Left, wrc.Top, wrc.Right-wrc.Left, wrc.Bottom-wrc.Top, TRUE);
//    ShowWindow(Handle, SW_SHOW);

    BitBlt (Canvas.Handle,0,0,ClientWidth,ClientHeight,b.Canvas.Handle,0,0,SRCCOPY);

    DeleteObject (hmyrgn);
    DeleteObject (hsumrgn);
    DeleteObject (htmprgn);
    DeleteObject (hrgn);
end;

procedure TfrmMain.GetBitmapFromBackground;
var
    MyIni           : TRegIniFile;
    a               : TBitmap;
    i,j             : integer;
    OffsetX,OffsetY : integer;

begin
    if BackBMPCreated then
        b.Free;
    b := TBitmap.Create;
    b.Width := ClientWidth;
    b.Height := ClientHeight;
    b.PixelFormat := pf24bit;
    MyIni := TRegIniFile.Create ('Control Panel');
    BackBMPFilename := MyIni.ReadString ('Desktop','Wallpaper','');
    BackBMPStyle    := MyIni.ReadInteger ('Desktop','WallpaperStyle',0);
    BackBMPTile     := MyIni.ReadInteger ('Desktop','TileWallpaper',0);
    if (BackBMPFilename = '') then
        Exit;
    if (FileExists (BackBMPFilename)) and (BackBMPStyle = 0) then
    begin
        a := TBitmap.Create;
        a.LoadFromFile (BackBMPFilename);
        case BackBMPTile of
        0 : begin
                DrawPartBMP (b.Canvas,0,0,b.Width,b.Height,0,0,a);
            end;
        1 : begin
                OffsetX := Left mod a.Width;
                OffsetY := Top mod a.Height;
                for i := 0 to (b.Width div a.Width)+1 do
                    for j := 0 to (b.Height div a.Height)+1 do
                        DrawPartBMP (b.Canvas,i*a.Width-OffsetX,j*a.Height-OffsetY,a.Width,a.Height,0,0,a);
            end;
        end;
        MakeFilter (Filter,b);

        b.Canvas.Pen.Color := clBlack;
        b.Canvas.MoveTo (0,0);
        b.Canvas.LineTo (b.Width-1,0);
        b.Canvas.LineTo (b.Width-1,b.Height-1);
        b.Canvas.LineTo (0,b.Height-1);
        b.Canvas.LineTo (0,0);
        a.Free;
        Canvas.Draw (0,0,b);
    end;
end;

procedure TfrmMain.FormPaint(Sender: TObject);
begin
    Canvas.Draw (0,0,b);
    if FirstTime then
    begin
        SetBkMode (Canvas.Handle,TRANSPARENT);
        Canvas.Font.Style := [fsBold];
        Canvas.Font.Color := clLime;
        Canvas.TextOut (8,8,'Right-Click for the menu');
        FirstTime := false;
    end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
    BackBMPCreated := false;
    MBDown := false;
    memDC := CreateCompatibleDC (frmMain.Canvas.Handle);
    Filter := 0;
    FirstTime := true;
    ClientWidth := 256;
    ClientHeight := 256;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
    GetBitmap;
end;

procedure TfrmMain.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
const
    SC_DRAGMOVE = $F012;

begin
    if Button = mbLeft then
    begin
        MBDown := true;
        MBX := X;
        MBY := Y;
    end;
end;

procedure TfrmMain.FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
begin
    if MBDown then
        MoveWindow (Handle,Left+X-MBX,Top+Y-MBY,Width,Height,true);
end;

procedure TfrmMain.FormMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    MBDown := false;
    if Button = mbLeft then
        GetBitmap;
end;

procedure TfrmMain.cmNormalClick(Sender: TObject);
begin
    (Sender as TMenuItem).Checked := true;
    Filter := (Sender as TMenuItem).Tag;
    GetBitmap;
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
    GetBitmap;
end;

procedure TfrmMain.cmAboutClick(Sender: TObject);
begin
    ShowMessage ('Transparent v1.1'#13#10+
                 '2000 CyLog Software'#13#10+
                 'written by K.Symeonidis && I.Tsoukalidis');
end;

procedure TfrmMain.cmExitClick(Sender: TObject);
begin
    Close;
end;

procedure TfrmMain.cmSelectColorClick(Sender: TObject);
begin
    dlgColor.Execute;
    GetBitmap;
end;

procedure TfrmMain.cmTintClick(Sender: TObject);
begin
    cmTint.Checked := not cmTint.Checked;
    GetBitmap;
end;

procedure TfrmMain.cm3DBorderClick(Sender: TObject);
begin
    cm3DBorder.Checked := not cm3DBorder.Checked;
    GetBitmap;
end;

procedure TfrmMain.cmIncreaseWidthClick(Sender: TObject);
begin
    ClientWidth := ClientWidth+32;
    GetBitmap;
end;

procedure TfrmMain.cmDecreaseWidthClick(Sender: TObject);
begin
    if ClientWidth > 32 then
    begin
        ClientWidth := ClientWidth-32;
        GetBitmap;
    end;
end;

procedure TfrmMain.cmIncHeightClick(Sender: TObject);
begin
    ClientHeight := ClientHeight+32;
    GetBitmap;
end;

procedure TfrmMain.cmDecHeightClick(Sender: TObject);
begin
    if ClientHeight > 32 then
    begin
        ClientHeight := ClientHeight-32;
        GetBitmap;
    end;
end;

end.
